#!/bin/env -S deno run -A

import { assert } from "https://deno.land/std@0.222.1/assert/assert.ts";
import { crayon } from "https://deno.land/x/crayon@3.3.3/mod.ts";
import { is_relevant } from "./lib/crypto.ts";
import { ApiClient } from "./lib/client.ts";
import { parallel, sum } from "./lib/utils.ts";
import { merge_request_parser, MergeRequest, note_parser } from "./lib/parser.ts";

async function main() {
	let { token } = parse_args(Deno.args);
	if (token === undefined) token = prompt_args();
	const client = new ApiClient("gitlab.cri.epita.fr", token);
	const merge_requests = get_all_merge_requests(client);
	const total = sum(await parallel(merge_requests, (mr) => count_in_merge_request(mr, client)));
	console.log("Total", total, "oki");
}

function parse_args(args: string[]) {
	const [token_part] = args;
	const token = token_part as string | undefined;
	return { token };
}

function prompt_args() {
	const [bl, gr] = [crayon.blue, crayon.lightBlack];
	const message = `
${bl("Please enter a read-allowed gitlab token :")}
${gr("You can get one from https://gitlab.cri.epita.fr/-/user_settings/personal_access_tokens")}
${gr("with at least the permission 'read_api', format is :")}
${gr("> xxxxxxxxxxxxxxxxxxxx")}
>`.trim();
	const result = prompt(message);
	assert(result !== null);
	return result.trim();
}

function get_all_merge_requests(client: ApiClient) {
	return client.fetch_parse_paginated(`api/v4/merge_requests`, merge_request_parser);
}

async function count_in_merge_request(merge_request: MergeRequest, client: ApiClient) {
	const all_notes = get_all_merge_request_notes(merge_request, client);
	let total = 0;
	for await (const note of all_notes) {
		if (!await is_relevant(note.author.username)) continue;
		if (note.body.toLowerCase().includes("oki")) total += 1;
	}
	return total;
}

function get_all_merge_request_notes(merge_request: MergeRequest, client: ApiClient) {
	const merge_request_path = `api/v4/projects/${merge_request.project_id}/merge_requests/${merge_request.iid}`;
	return client.fetch_parse_paginated(`${merge_request_path}/notes`, note_parser);
}

if (import.meta.main) await main();
